function y = findBMParams(param,PutCall,S0,K,r,q,T,N,MktPrice)

% Objective function to find the parameter estimates in the Brigo,
% Mercurio and Rapidsada lognormal mixture model.

% by Fabrice Douglas Rouah, FRouah.com and Volopta.com

% First N-1 weights
for k=1:N-1
    L(k) = param(k);
end

% Last weight
complement = sum(L);
L(N) = 1 - complement;

% beta parameters
for k=N:2*N-1
    beta(k-N+1) = param(k);
end

% a parameters
for k=2*N:3*N-1
    a(k-2*N+1) = param(k);
end

% b parameters
for k=3*N:4*N-1
    b(k-3*N+1) = param(k);
end

% c parameters
for k=4*N:5*N-1
    c(k-4*N+1) = param(k);
end

% tau parameters
for k=5*N:6*N-1
    tau(k-5*N+1) = param(k);
end

% Indicator for put or call
if strcmp(PutCall,'C')
    w = 1;
elseif strcmp(PutCall,'P')
    w = -1;
end

NK = length(K);
NT = length(T);

for k=1:NK;
    for t=1:NT;
        for i=1:N
            % Integrated volatilities
            eta(t,i) = a(i) + b(i)*(1-exp(-T(t)/tau(i)))*(tau(i)/T(t)) ...
                     + c(i)*exp(-T(t)/tau(i));
            LSK = log((S0-beta(i))/K(k));
            % Make sure the eta are positive
            if eta(t,i) <= 0
                y = 1e100;
            end
            % make sure the logarithms are defined
            if isreal(LSK)
                d1 = w*(LSK + (r-q+0.5*eta(t,i)^2)*T(t))/eta(t,i)/sqrt(T(t));
                d2 = w*(LSK + (r-q-0.5*eta(t,i)^2)*T(t))/eta(t,i)/sqrt(T(t));
                Option(i) = L(i)*((S0-beta(i))*exp((r-q)*T(t))*normcdf(d1) - K(k)*normcdf(d2));
            else
                Option(i) = 1e100;
            end
        end
        % Model price and error
        ModelPrice(k,t) = w*exp(-r*T(t))*sum(Option);
        error(k,t) = (ModelPrice(k,t) - MktPrice(k,t))^2 / MktPrice(k,t)^2;
    end
end

% Sum of errors
y = sum(sum(error));

% Penalty for inadmissable weights
for k=1:N
    if L(k)<0.01 || L(k)>0.99;
        y = 1e100;
    end
end

% Ensure no arbitrage across integrated volatilities
for k=1:N
    for t=1:NT-1
        if eta(t+1,i) <= eta(t,i)*sqrt(T(t)/T(t+1))
            y = 1e100;
        end
    end
end

